# escape_tex.awk
#
# Escape TeX-special constructions and convert HTML-constructs to LaTeX.
# General functions dealing with BibTeX entries.
#
# Notes:
# * This is an awk library.
#
# Programmed with gawk 3.1.1
#
# Copyright (C) by Volker Kuhlmann <VolkerKuhlmann@gmx.de>
# Released under the terms of the GNU General Public License (GPL) Version 2.
# See http://www.gnu.org/ for details.
#
# Volker Kuhlmann
#  16, 17, 18, 19, 20 Feb; 1, 2, 21 Mar 2003
#

# Deal with TeX's special characters in the given fields.
# (Fields are passed as space-separated names in one only string.)
# Note: correction of space between numbers and units doesn't always work
# Note: only some of the HTML special character names are implemented
# Note: this will save some, but not all, conversion work
function escape_tex(entry, fields   	    ,f,ff) {
    split(fields, ff, " ")
    for (f in ff) {
    	## backslash, ampersand, and a few others
	gsub("\\\\", "\\backslash", entry[ff[f]]) # can't put \backslash{} here
	gsub("[{}^_%]", "\\\\&{}", entry[ff[f]])
    	gsub("~", "\\ensuremath{\\approx}", entry[ff[f]])
	entry[ff[f]] = translate_html(entry[ff[f]])
	gsub("[<>]", "\\ensuremath{&}", entry[ff[f]])
	
	## units
	# HZ
	entry[ff[f]] = gensub("([0-9]) *([pnmckMGT])Hz",
	    	    	    	"\\1\\\\,\\2Hz", "g", entry[ff[f]])
	entry[ff[f]] = gensub("([0-9]) *Hz", "\\1\\\\,Hz", "g", entry[ff[f]])
	# dB
	entry[ff[f]] = gensub("([0-9]) *dB",
	    	    	    	"\\1\\\\,dB", "g", entry[ff[f]])
	# W, A, V, m, s
	entry[ff[f]] = gensub("([0-9]) *([pnmckMGT])([WAVms])([.,?!)}/ ])",
	    	    	    	"\\1\\\\,\\2\\3\\4", "g", entry[ff[f]])
	entry[ff[f]] = gensub("([0-9]) *([WAVms])([.,?!)}/ ])",
	    	    	    	"\\1\\\\,\\2\\3", "g", entry[ff[f]])
    }
}

function translate_html(string) {
    	#gsub("&mu;", "\\ensuremath{\\mu}", string)
    	gsub(" *&mu; *", "\\muunit{}", string)
    	gsub("&Omega;", "\\ensuremath{\\Omega}", string)
	gsub("&deg;C", "\\celsius{}", string)
	gsub("&deg;", "\\degree{}", string)
	gsub("&middot; *", "\\ensuremath{*}", string)
	gsub("&times; *", "\\ensuremath{*}", string)
	gsub("&tau;", "\\ensuremath{\\tau}", string)
	gsub("&delta;", "\\ensuremath{\\delta}", string)
	gsub("&sigma;", "\\ensuremath{\\sigma}", string)
	gsub("&plusmn;", "\\ensuremath{\\pm}", string)
	gsub("&ldquo;", "``", string)
	gsub("&rdquo;", "''", string)
	gsub("&amp;", "\\&", string)
	gsub("&Aring;", "", string)
	gsub("&nbsp;", "~", string)
	gsub("&lt;", "\\ensuremath{<}", string)
	gsub("&gt;", "\\ensuremath{>}", string)
	string = gensub("&radic;\\(([^)]*)\\)",
	    	    	"\\\\ensuremath{\\\\sqrt{}{\\1}}", "g", string)
	gsub("&radic;", "\\ensuremath{\\sqrt{}{FIXME}}", string)
	gsub("[&]", "\\\\&{}", string)
	string = gensub("<e[0-9]>([^<]*)</e[0-9]>",
	    	    	    	"\\\\emph{\\1}", "g", string)
	string = gensub("<sup>TM</sup>",
	    	    	    	"\\\\texttrademark{}", "g", string)
	string = gensub("<sup>([^<]*)</sup>",
	    	    	    	"\\\\ensuremath{^{\\1}}", "g", string)
	string = gensub("<sub>([^<]*)</sub>",
	    	    	    	"\\\\ensuremath{_{\\1}}", "g", string)
	return string
}
