# fix-ieeeris.awk
#
# Fix up broken citation entries which were exported by the IEEExplore data
# base (http://ieeexplore.ieee.org/).
#
# Notes:
# * You need to run this also for importing IEEE citation records into endnote.
#
# Programmed with gawk 3.1.1
#
# Copyright (C) by Volker Kuhlmann <VolkerKuhlmann@gmx.de>
# Released under the terms of the GNU General Public License (GPL) Version 2.
# See http://www.gnu.org/ for details.
#
# Volker Kuhlmann
#  16, 18, 27 Feb; 29 Apr 2003
#

# Change stuffed volume numbers in records:
# From:
#   VO  -
#   SP  - 1878
#   EP  - 1882 vol.3
# To:
#   VO  - 3
#   SP  - 1878
#   EP  - 1882
# 
# From:
#   VO  - 1
#   SP  - 16
#   EP  - 20 vol.9
#   VL  - 1
# To:
#   SP  - 16
#   EP  - 20
#   VO  - 1 -?which one?- 9
#   VL  - 1 -?which one?- 9

BEGIN {
    # remove whitespace and CR from beginning and end of line
    RS="[ \t]*\r*[ \t]*\n\r*[ \t]*"
    ORS="\n"
}

function zapvars() {
    volume=""
    newvolume=""
    pagevol=""
    fix=0
    pagefields=0
}

# Save the VO field
$0 ~ /^VO / {
    volume=$0
    fix=1
    next
}

# Adjust the VL field the same as VO, if appropriate,
# and deal with pagevol if only one of SP/EP was found.
# (This assumes VL occurs after VO, SP and EP - so far no record to the
# contrary found.)
$0 ~ /^VL / {
    if ( substr($0,3) == substr(volume,3)  &&  newvolume != "" )
        $0 = "VL" substr(newvolume,3)
    if ( pagefields == 1 ) {
	pagefields=0
	addtovol()
    }
    if ( fix ) 
    	print volume
    zapvars()
}

# Split the volume number off the EP or SP field and save it; print SP/EP
# (If both SP and EP have a volume number the first saved is lost)
$0 ~ /^[SE]P / {
    pagefields += 1
    fields = split ($0, endpage, " *[Vv][Oo][Ll]\\.")
    # print SP/EP up to before "vol." (or all if there is no "vol.")
    print endpage[1]
    # save volume number from page field, if any
    if (fields > 1) pagevol = endpage[2]
    next
}

# add the page volume number to the volume field, and print volume field
function addtovol(   	    	oldvolume) {
    if ( pagevol == "" ) {
    	if ( fix )
	    print volume
    } else {
	if (match(volume, "^[Vv][Oo] *- *$")) {
	    # VO is empty
	    print volume " " pagevol
	} else {
	    # VO is not empty - if new value is different from old, add
	    # new to old value with a warning
	    oldvolume = volume
	    sub("^..[0-9]*[^0-9]*", "", oldvolume)
	    sub("[ \t]*$", "", oldvolume)
	    if ( pagevol != oldvolume )
		newvolume = volume " -?which one?- " pagevol;
	    else
		newvolume = volume
	    print newvolume
	}
    }
    fix=0
}

# when both SP and EP were read, deal with volume
pagefields >= 2 {
    pagefields=0
    addtovol()
}

# Safety stop, if there's no EP field.
# Also clear some variables at end of record, in case of multiple records in
# input.
# (This assumes these fields are after VO, VL, SP and EP)
#fix && $0 ~ /^KW / { print volume; zapvars() }  # KW is before VL
fix && $0 ~ /^AB / { print volume; zapvars() }
fix && $0 ~ /^ER / { print volume; zapvars() }

# Print current line
{ print }
