# ieee_names.awk
#
# Convert citations exported from IEEE (they call the format "ISI Researchsoft")
# to BibTeX. The format is generated by http://ieeexplore.ieee.org/ in Feb 2003.
# The format is similar, but not identical, to RIS.
#
# Requires:
#
# Notes:
# * This is an awk library.
# * The list of IEEE names is highly incomplete.
#
# Programmed with gawk 3.1.1
#
# Copyright (C) by Volker Kuhlmann <VolkerKuhlmann@gmx.de>
# Released under the terms of the GNU General Public License (GPL) Version 2.
# See http://www.gnu.org/ for details.
#
# Volker Kuhlmann
#  16, 17, 18, 19, 20, 21, 28 Feb; 1, 2, 6, 13, 21, 26 Mar; 12 Apr 2003
#

## Correct some of the journal names
function correct_journal_names(entry, field            ,ic) {
    ic=IGNORECASE
    IGNORECASE=1

    gsub("Computing in Science & Engineering",
    	    "Computing in Science and Engineering", entry[field])
    gsub("IEEE Security & Privacy Magazine",
    	    "IEEE Security and Privacy Magazine", entry[field])
    gsub("IEEE Instrumentation & Measurement Magazine",
    	    "IEEE Instrumentation and Measurement Magazine", entry[field])

    IGNORECASE=ic
}

## IEEE transaction names and BibTeX shortcuts
# (This initiliases a global variable)
function init_IEEE_names() {
    IEEE_trans["Communications"]="IEEE_J_COM"
    IEEE_trans["Power Delivery"]="IEEE_J_PWRD"
    IEEE_trans["Power Systems"]="IEEE_J_PWRS"
    IEEE_trans["Instrumentation and Measurement"]="IEEE_J_IM"
    IEEE_trans["Instrumentation & Measurement"]="IEEE_J_IM"
    IEEE_trans["Computers"]="IEEE_J_C"
    IEEE_trans["Networking"]="IEEE_J_NET"
    IEEE_trans["Ultrasonics, Ferroelectrics and Frequency Control"]="IEEE_J_UFFC"
    IEEE_trans["Ultrasonics Ferroelectrics and Frequency Control"]="IEEE_J_UFFC"
    IEEE_trans["Automatic Control"]="IEEE_J_AC"
    IEEE_trans["Nuclear Science"]="IEEE_J_NS"
    IEEE_trans["Circuits and Systems II: Analog and Digital Signal Processing"]="IEEE_J_CASII"
    IEEE_trans["Circuits and Systems II-Analog and Digital Signal Processing"]="IEEE_J_CASII"
    IEEE_trans["Parallel and Distributed Systems"]="IEEE_J_PDS"
    IEEE_trans["Aerospace and Electronic Systems"]="IEEE_J_AES"
}

## Try and translate some of the IEEE journal names into BibTeX strings/macros.
# (This also calls correct_journal_names() )
function translate_journal_names(entry, field           ,journal,ic) {
    ic=IGNORECASE
    IGNORECASE=1

    init_IEEE_names()
    correct_journal_names(entry, field)
    
    for (journal in IEEE_trans) {
    	gsub ("IEEE Transactions on " journal,
	    IEEE_trans[journal], entry[field])
    	gsub (journal ", IEEE Transactions on",
	    IEEE_trans[journal], entry[field])
    }
    
    journal = entry[field]

    # IEEE Journals with a name other than "IEEE Transactions on .."
    gsub ("IEEE/ACM Transactions on Networking", "IEEE_J_NET", journal)
    gsub ("Networking, IEEE/ACM Transactions on", "IEEE_J_NET", journal)
    gsub ("IEEE Journal on Selected Areas in Communications", "IEEE_J_JSAC", journal)
    gsub ("Selected Areas in Communications, IEEE Journal on", "IEEE_J_JSAC", journal)

    # IEEE Magazines
    gsub ("IEEE Instrumentation (and|&) Measurement Magazine", "IEEE_M_IM", journal)
    gsub ("IEEE Aerospace (and|&) Electronics Systems Magazine", "IEEE_M_AES", journal)
    gsub ("IEEE Antennas (and|&) Propagation Magazine", "IEEE_M_AP", journal)
    gsub ("Computing in Science (and|&) Engineering \\[see also IEEE Computational Science (and|&) Engineering\\]", "IEEE_M_CSE", journal)
    gsub ("IEEE Control Systems Magazine", "IEEE_M_CS", journal)
    gsub ("IEEE Power Engineering Review", "IEEE_M_PER", journal)
    gsub ("IEEE Computer Applications in Power", "IEEE_M_CAP", journal)
    gsub ("IEEE Power (and|&) Energy Magazine", "IEEE_M_PE", journal)
    gsub ("IEEE Security (and|&) Privacy Magazine", "IEEE_M_SP", journal)
    #
    gsub ("Proceedings of the IEEE", "IEEE_J_PROC", journal)
    gsub ("IEEE Micro", "IEEE_M_MICRO", journal)
    gsub ("Micro, IEEE", "IEEE_M_MICRO", journal)
    gsub ("IEEE Spectrum", "IEEE_M_SPECT", journal)
    gsub ("IEEE Potentials", "IEEE_M_POT", journal)

    # IEE Journals / Proceedings:
    gsub ("IEE Proceedings-Radar Sonar and Navigation", "IEE_P_RSN", journal)
    # not current:
    gsub ("Engineering Science and Education Journal", "IEE_J_ESE", journal)
    gsub ("Power Engineering Journal", "IEE_J_PEJ", journal)

    # IEE Magazines:
    gsub ("Power Engineer", "IEE_M_PE", journal)
    gsub ("IEE Review", "IEE_M_REV", journal)
    
    entry[field] = journal
    IGNORECASE=ic
}

## Try and translate some of the IEEE conference names into useful order
function translate_conf_names(entry, field         ,conf) {
    conf = entry[field]
    
    conf = gensub(\
    "(Frequency Control, [^,]*), (Proceedings of the [^ ]* Annual Symposium on)",\
    "\\2 \\1", "", conf)
    conf = gensub(\
    "(Robotics and Automation, .*), ([0-9]* {IEEE} International Conference on)",\
    "\\2 \\1", "", conf)
    conf = gensub(\
    "(Precision Electromagnetic Measurements, .*), (Conference on)",\
    "\\2 \\1", "", conf)
    conf = gensub(\
    "(Real-Time Systems Symposium, .*), (The .* {IEEE})",\
    "\\2 \\1", "", conf)
    conf = gensub(\
    "(Instrumentation and Measurement Technology .*) (Proceedings of the .*)",\
    "\\2 \\1", "", conf)
    
    entry[field] = conf
}
